@@ -1,22 +1,23 @@
 /*
- * libgail-gnome
- * Copyright 2002 Sun Microsystems Inc.
+ * LIBGAIL-GNOME -  Accessibility Toolkit Implementation for Bonobo
+ * Copyright 2002 - 2004 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
+ * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
+ * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
 #include <stdio.h>
 #include <glib-object.h>
 #include <atk/atk.h>
@@ -25,13 +26,19 @@
 #include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-control-frame.h>
+#include <panel-applet.h>
 #include <libspi/accessible.h>
 #include "bonobo-control-accessible.h"
 #include "bonobo-control-frame-accessible.h"
 #include "bonobo_plug_atk_object_factory.h"
 #include "bonobo_socket_atk_object_factory.h"
+#include "panel-applet-atk-object-factory.h"
 #include "gail-gnome-debug.h"
 
+#define GNOME_ACCESSIBILITY_ENV "GNOME_ACCESSIBILITY"
+
+G_MODULE_EXPORT int gtk_module_init (gint *argc, char** argv[]);
+
 /* For automatic libgnome init */
 extern void gnome_accessibility_module_init     (void);
 extern void gnome_accessibility_module_shutdown (void);
@@ -46,6 +53,7 @@ impl_bonobo_control_frame_get_parent_accessible (PortableServer_Servant servant,
   BonoboControlFrame *control_frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
   AtkObject          *atko;
   AtkObject          *parent;
+  SpiAccessible      *co = NULL;
 
   dprintf ("Bonobo::ControlFame::getParentAccessible [%p]\n", control_frame);
 
@@ -54,10 +62,11 @@ impl_bonobo_control_frame_get_parent_accessible (PortableServer_Servant servant,
   atko = gtk_widget_get_accessible (
 	  bonobo_control_frame_get_widget (control_frame));
 
-  parent = atk_object_get_parent (atko);  
+  parent = atk_object_get_parent (atko);
 
-  return CORBA_Object_duplicate (
-	  BONOBO_OBJREF (spi_accessible_new (parent)), ev);
+  if (parent) co = spi_accessible_new (parent);
+
+  return (co != NULL) ? CORBA_Object_duplicate (BONOBO_OBJREF (co), ev) : CORBA_OBJECT_NIL;
 }
 
 static Bonobo_Unknown
@@ -65,13 +74,17 @@ impl_bonobo_control_get_accessible (PortableServer_Servant servant,
 				    CORBA_Environment *ev)
 {
   BonoboControl *control = BONOBO_CONTROL (bonobo_object_from_servant (servant));
+  BonoboControlAccessible *co = NULL;
 
   dprintf ("Bonobo::Control::getAccessible [%p]\n", control);
 
   g_return_val_if_fail (control != NULL, CORBA_OBJECT_NIL);
   
-  return CORBA_Object_duplicate (
-	  BONOBO_OBJREF (bonobo_control_accessible_new (control)), ev);
+  co = bonobo_control_accessible_new (control);
+
+  if (co != NULL) g_assert (BONOBO_IS_CONTROL_ACCESSIBLE (co));
+
+  return (co != NULL) ? CORBA_Object_duplicate (BONOBO_OBJREF (co), ev) : CORBA_OBJECT_NIL;
 }
 
 static void
@@ -80,6 +93,7 @@ gail_bonobo_ui_register_atk_factories (void)
   AtkRegistry *registry = atk_get_default_registry ();	
   atk_registry_set_factory_type (registry, BONOBO_TYPE_PLUG, BONOBO_TYPE_PLUG_ATK_OBJECT_FACTORY);
   atk_registry_set_factory_type (registry, BONOBO_TYPE_SOCKET, BONOBO_TYPE_SOCKET_ATK_OBJECT_FACTORY);   
+  atk_registry_set_factory_type (registry, PANEL_TYPE_APPLET, PANEL_APPLET_TYPE_ATK_OBJECT_FACTORY);
 }
 
 static void
@@ -111,7 +125,7 @@ gail_accessibility_init (void)
   gail_bonobo_ui_register_atk_factories (); 
 }
 
-int
+G_MODULE_EXPORT int
 gtk_module_init(gint *argc, char** argv[])
 {
   dprintf ("BonoboUI Accessibility Module loaded\n");
@@ -124,7 +138,8 @@ gtk_module_init(gint *argc, char** argv[])
 void
 gnome_accessibility_module_init (void)
 {
-  fprintf (stderr, "Bonobo accessibility support initialized\n");
+  if (g_getenv (GNOME_ACCESSIBILITY_ENV))
+    fprintf (stderr, "Bonobo accessibility support initialized\n");
   gail_accessibility_init ();
 }
 
